#! /usr/bin/env python

import sys, string, random, numarray, math
import xmlReader

#**************************** class Cell ****************************************"

class Cell(object):

	"""holds info about a single cell 

	for each cell store the value, if determined 
	a list of all possible values
	a list of the groups of which it is part"""

	digits = string.digits + string.ascii_lowercase

	#********************** standard methods *******************"

	def __init__(self, groups={"row": None, "col": None, "square": None}, grid=None):
		self.__value = 0 
		self.possibles = [] 
		self.groups = groups
		self.grid = grid
		self.__number = None

	def __str__(self):
		n = self.number
		r = n/(self.size**2) + 1
		c = n%(self.size**2) + 1
		return "cell(%d,%d)" % (r,c)

	#****************** properties ****************"

	def setValue(self, value, updateSolve=True):
		"""sets the value of the cell

		by default the Grid.solve() variables depending on this cell are updated
		to reflect the new value"""
		if type(value) == tuple: (value, updateSolve) = value
		if type(value) == int:
			self.__value = value
		else:
			self.__value = self.digits.index(value)
		if updateSolve: 
			self.possibles = [value]
			[s.removeReqs(value) for s in self.groups]
		return True # required by Grid.checkGroup()

	def __getValue(self): return self.__value
	value = property(__getValue, setValue)
	
	def __getNumber(self):
		if self.__number != None: return self.__number
		self.__number = self.grid.cells.index(self)
		return self.__number
	number = property(__getNumber)

	def __getRow(self):
		"returns the ordinal number of the cells row"
		return self.grid.groups.index(self.groups[0])
	row = property(__getRow)

	def __getCol(self):
		"returns the ordinal number of the cells column"
		return self.grid.groups.index(self.groups[1]) % self.size**2
	col = property(__getCol)

	def __getSquare(self):
		"returns the ordinal number of the cells square"
		return self.grid.groups.index(self.groups[2]) % self.size**2
	square = property(__getSquare)

	def __getSize(self):
		return self.grid.size
	size = property(__getSize)
	#********************** class methods *******************"
	
	def alreadyGot(cls, list, value): 
		"""returns whether the cells in the list contain the value

		since in this case we know the value is chosen from the
		list of unmet requirements there is no need to check we
		know the answer is always false"""
		return False
	alreadyGot = classmethod(alreadyGot)

	#********************** utility methods *******************"
	
	def store(self):
		"""makes a copy of information about the current state of the cell

		returns a tuple of the appropriate data"""
		value = self.__value
		possibles = self.possibles[:]
		return (value, possibles)
	
	def restore(self, storeCell):
		"restores the cell to the state captured by the store method"
		self.__value = storeCell[0]
		self.possibles = storeCell[1][:]

	def printValue(self):
		"converts the value to a letter (if its greater than 9)"
		return self.digits[self.__value]

	#********************** solving methods *******************"

	def initializeSolveValues(self):
		"""sets inital values for Grid.solve() 
		
		must be called after Groups.initializeSolveValues()"""
		self.possibles = range(1,self.size*self.size+1)
		if self.__value:
			self.possibles = [self.__value]
			for s in self.groups: s.removeReqs(self.__value)

	def reducePossibles(self):
		"removes from the cell's possibles list the values of other cells in the same groups"
		for s in self.groups:
			for c in s.cells:
				try: self.possibles.remove(c.value) 
				except ValueError: pass

	def checkLoneValue(self, setit = True):
		"""checks if there's only one possible value for the cell
		
		if there is only one possible value, and setit is True, then
		this method sets the value.
		raises ImpossibleGridError if there are no possible values
		returns 1  if the value was already set
				-1 if the value was set in this call (or could have been)
				0  if the value is not yet known"""
		if self.__value: return 1

		self.reducePossibles()
	
		if len(self.possibles) == 1:
			if setit:
				self.value = self.possibles[0]
				self.grid.reportCell(self)
			return -1
		elif not len(self.possibles):
			message = "no possible values for %s" % self
			if self.grid.report:
				self.grid.report.message(message)
			raise ImpossibleGridError(message)
		return 0

	def isRedundant(self):
		"""try setting the cell to zero, see if there's a unique solution

		This algorithm finds all the cells which could be removed
		without affecting the solution, although they cannot all be removed
		simultaneously"""

		if self.__value == 0: return False
		restore = self.grid.listValues()
		self.__value = 0
		numSols = self.grid.solve(2)
		self.grid.restoreValues(restore)
		return numSols == 1

#**************************** class Group*******************************"

class Group(object):
	"holds info about a row, column, or square of the grid"
	
	def __init__(self, cells=None, grid=None):
		"""for each group store a list of the cells of which it is comprised
		and a list of the values yet to be assigned to a cell"""
		self.cells = cells
		self.requires = []
		self.grid = grid
		self.__number = None

	def __str__(self):
		n = self.number
		if n < self.size*self.size: return "row %d" % (n + 1)
		elif n < 2*self.size*self.size: return "col %d" % (n - self.size*self.size + 1)
		else: return "box %d" % (n - 2*self.size*self.size + 1)

	#****************** properties ****************"

	def __getNumber(self):
		if self.__number != None: return self.__number
		self.__number = self.grid.groups.index(self)
		return self.__number
	number = property(__getNumber)

	def __getSize(self):
		return self.grid.size
	size = property(__getSize)

	#********************** utility methods *******************"
	def store(self):
		"""makes a copy of information about the current state of the group

		returns a sequence of the appropriate data"""
		requires = self.requires[:]
		return requires

	def restore(self, storeGroup):
		"restores the group to the state captured by the store method"
		self.requires = storeGroup[:]

	#********************** solving methods *******************"

	def initializeSolveValues(self):
		self.requires = range(1,self.size*self.size+1)
	
	def removeReqs(self, value):
		"""removes the value from the group.requires list

		returns True if the value was present, False otherwize
		does not fail if the value is not present"""
		try:
			self.requires.remove(value)
			return True
		except ValueError:
			return False

#**************************** class Grid ****************************************"

class Grid(object):
	"""holds information about the entire grid

	cells are numbered 0-80 left to right, then top to bottom
	groups are numbered as follows:
		rows: 0-8 top to bottom
		columns: size*size - 2*size*size-1 left to right
		squares: 2*size*size - 3*size*size-1 left to right, then top to bottom"""

	#********************** standard methods *******************"
	
	def __init__(self, size=3, file=None, string='', xmlFile=None, csvFile=None, cursor=None, id=None,
				table='puzzle', grid=None, array=None, blank=False):
		"""define either size, or file or string or array which is a numarray.Array object or grid) """
		#create a new grid
		self.reader = None
		self.report = DisplayReport(self) #object which controlls reporting
		self.recorder = None
		#variables used in finding a solution
		self.__guess = None #controls alternative branches explored
		self.__alternatives = 0
		self.solving = False
		self.__restoreOnFail = []
		self.nsolutions = 0
		self.solutions = []
		self.cells = []

		self.__size = size  # may be overridden by setReader() or direct assignment below
		if grid:
			self.__size = grid.size
		elif array != None:  # must use != None here!
			self.__size = int(math.sqrt(len(array)))
		# following alternative sets the size when data comes from a file or string
		elif not blank and file <> "BLANK":
			self.setReader(file=file, string=string, xmlFile=xmlFile, csvFile=csvFile)
	
		self.groups = [
			self.specifyGroup()(None, self) for i in range((3*self.size*self.size))]
		self.__setStandingData()

		if grid:
			self.restoreValues(grid.listValues())
		elif array != None: #need != None here
			self.fromArray(array)
		
		if self.reader: self.read()

	#************* properties *************"
	
	def __getSize(self): return self.__size
	size = property(__getSize)

	#************* static methods *********"

	def specifyGroup():
		"specifies the class to be used for groups"
		return Group
	specifyGroup = staticmethod(specifyGroup)
	
	def specifyCell():
		"specifies the class to be used for cells"
		return Cell
	specifyCell = staticmethod(specifyCell)

	def getIndex(row, col, size): # look no self!
		"""returns the location in the grid of the cell identified by row and col

		note that the col number is the ordinal number of the column, 
		starting at zero, it is not the same as the group number for the column"""
		return row*size*size + col
	getIndex = staticmethod(getIndex)

	#************* utility methods *********"
	def index(self,row,col): return self.__class__.getIndex(row,col,self.size)

	def findZeros(self):
		return [self.cells.index(cell) 
					for cell in self.cells if not cell.value]

	def count(self, silent=False):
		"counts the numbers of set and unset cells"
		unset = [cell.value for cell in self.cells].count(0)
		set = len(self.cells) - unset
		if not silent and self.report and self.report.do(1):
			self.report.message(("%d cells have values, %d are unset" % 
									(set, unset)),1)
		return set
		
	def setValue(self, row, col, value, update=True):
		"sets the value of the cell given its row and colum"
		self.cells[self.index(row,col)].value = (value,update)
		if update and self.recorder: self.recorder.store()

	def cell(self, row, col):
		"returns the cell at the specified row and column position (zero-based)"

		return self.cells[self.index(row, col)]

	def listValues(self):
		"saves the values so they can be restored by restoreValues"
		return [cell.value for cell in self.cells]

	def restoreValues(self, list):
		"restores the values saved by listValues"
		for i in range(len(list)): self.cells[i].value = list[i] 
		if self.recorder: self.recorder.store()

	def zeroValues(self):
		for i in range(len(self.cells)): self.cells[i].value = 0
		if self.recorder: self.recorder.store()

	def store(self): return StoredGrid(self)
	
	def restore(self, storedGrid):
		"restores the grid to the stored state captured by store"

		[self.groups[i].restore(storedGrid.groups[i]) 
							for i in range(len(self.groups))]
		[self.cells[i].restore(storedGrid.cells[i]) 
							for i in range(len(self.cells))]

	def applySkeleton(self, skeleton):
		for i in range(len(self.cells)):
			self.cells[i].value = skeleton.listValues()[i] and self.cells[i].value or 0

	#***** methods for file or interactive io *****
	
	def printIt(self):
		"returns a print string for the grid suitable for output to a terminal"
		return ("\n".join([" %s"*self.size*self.size % 
						tuple([self.cells[c+self.size*self.size*r].printValue() 
						for c in range(self.size*self.size)]) 
						for r in range(self.size*self.size)]))

	def writeIt(self):
	
		"returns a print string for the grid suitable for output to a file"
		return ("['"+"', '".join([('%s'*self.size*self.size % 
						tuple([self.cells[i+self.size*self.size*j].printValue() 
						for i in range(self.size*self.size)])) 
						for j in range(self.size*self.size)])+"']\n")

	def writeCsv(self):
		"returns a comma separated print string for the grid suitable for output to a file"
		return ("\n".join([(','.join([self.cells[i+self.size*self.size*j].value 
							and ("%2d" % (self.cells[i+self.size*self.size*j].value)) or '  ' 
							for i in range(self.size*self.size)])) 
							for j in range(self.size*self.size)]) + "\n\n")

	def writeXml(self):
		return str(xmlReader.XmlWriter(self))

	def setReader(self, file=None, string='', xmlFile=None, csvFile=None):
		if xmlFile or csvFile or file or string: # in these cases set the size from the data
			if xmlFile: self.reader = xmlReader.XmlReader(file = xmlFile)
			elif csvFile: self.reader = InputReader(csvFile = csvFile)
			elif file: self.reader = InputReader(input = file)
			else: self.reader = InputReader(string = string)
			self.__size = self.reader.size
		else: # in this case use the size which was specified in the Grid initializer
			self.reader = InputReader(size = self.size)

	def read(self):
		"reads values from the terminal or file and sets the starting values of the grid"
		if isinstance(self.reader,xmlReader.XmlReader):
			self.reader.readCells(self)
		else:
			dataline = self.reader.dataLine()
			for i in range(self.size*self.size):
				data = dataline.next()
				for j in range(self.size*self.size):
					if data[j]: self.setValue(i, j, data[j], False)
		self.reader.close()
		if self.recorder: self.recorder.store()

	#********************** solving methods *******************"

	def __setStandingData(self):
		#define cells"
		self.cells = [self.specifyCell()
				((self.groups[r],self.groups[c+self.size*self.size],
				self.groups[(r/self.size)*self.size+c/self.size+2*self.size*self.size]),self) 
				for r in range(self.size*self.size) for c in range(self.size*self.size)]
		#inform the groups which cells are in them"
		for r in range(self.size*self.size):
			self.groups[r].cells = [self.cells[r*self.size*self.size+c] 
				for c in range(self.size*self.size)]
		for c in range(self.size*self.size):
			self.groups[self.size*self.size+c].cells = [self.cells[r*self.size*self.size+c] 
				for r in range(self.size*self.size)]
		for s in range(self.size*self.size):
			self.groups[2*self.size*self.size+s].cells = [self.cells[r*self.size*self.size+c] 
				for r in range(self.size*self.size) for c in range(self.size*self.size) 
				if (r/self.size)*self.size+c/self.size == s]
		
	def initializeSolveValues(self, resetCounters=True):
		for group in self.groups: group.initializeSolveValues()
		for cell in self.cells: cell.initializeSolveValues()
		for cell in self.cells: cell.checkLoneValue(False)
		if resetCounters:
			self.nsolutions = 0
			self.solutions = []
			self.__alternatives = 0
			self.__guess = None

	def checkGroup(self, group, individual="cells"):
		"""find an individual which is the only possible provider 
		of some value required by the group, and set it

		returns True  we have found a value to set
				False if no suitable cell was found
		raises ImpossibleGridError if there is no solution for this leaf"""

		for s in group:
			# for each group go through the list of values not yet used and check 
			#how many of the cells which comprise the group could take that value - 
			#if there is only one then set the value of the cell
			if s.requires:
				individuals = eval("s." + individual)
				for value in s.requires:
					if individuals[0].alreadyGot(individuals, value): continue
					valFound = [c.possibles.count(value) for c in individuals]
					if sum(valFound) == 0:
						message = ("no possible %s to provide value %d in %s" % 
							(individual, value, s))
						if self.report: self.report.message(message)
						raise ImpossibleGridError(message)
					if valFound.count(1) == 1:
						c = individuals[valFound.index(1)]
						if c.setValue(value):
							self.reportCellInGroup(c, s)
							return True
		return False

	def findRedundants(self, pos=0):
		"""returns a list of redundant cells starting at pos 

		uses the Cell.isRedundant() method to return a list of cells
		starting at pos whose value may be determined from the others.
		note that the cells are not redundant en masse, 
		only each one individually"""

		self.report.on = False
		retVal = [self.cells.index(cell) for cell in self.cells[pos:] 
									if cell.value and cell.isRedundant() ]
		self.report.on = True
		return retVal

	def reportCell(self, cell, guess=False):
		"""reports that a value has been asigned to a cell
		
		because all other values had been eliminated from the list of
		possiblities.  the actual output is controlled by instance variables
		verbose and printGrid"""

		if not self.report: return
		self.report.display(4)
		if self.report.do(3):
			self.report.message("%s set to %d %s" % 
				(cell, cell.value, 
				(guess) and "(guess)" or "(no other values were possible)"),
				3, linenum=True)

	def reportCellInGroup(self, cell, group):
		"""reports that a value has been asigned to a cell
		
		because no other cell could provide that value for a group.  the actual
		output is controlled by instance variables verbose and printGrid"""

		if not self.report: return
		self.report.display(4)
		if self.report.do(3):
			self.report.message( "%s set to %s "
				"(no other cell in %s could have that value)" % 
				(cell, cell.value, group), 3, linenum=True)

	def solve(self, stopAt = 0):
		"""finds all solutions for the grid, reporting as it goes

		If stopAt is non zero the solution stops after that number
		of solutions has been found
		the amount of detail in the report is controlled by instance variables
		verbose and printGrid"""
		if not self.solving:
			self.initializeSolveValues()
			self.solving = True
			self.__restoreOnFail = self.listValues()
		#go through the cells, for each one eliminating the values of other cells 
		#in each of the groups of which it is part
		try:
			modified = \
				[cell.checkLoneValue() for cell in self.cells].count(-1) 

			# for each group go through the list of values not yet used and check 
			#how many of the cells which comprise the group could take that value - 
			#if there is only one then set the value of the cell
			if self.checkGroup(self.groups):
				#we have just set a value - go back to checking cells"
				return self.solve(stopAt)
			elif [cell.value for cell in self.cells].count(0) == 0:
				#...we have found a solution 
				#- print it and contiue to look for alternatives
				self.nsolutions += 1
				if self.report and self.report.do(2):
					self.report.message("solved:",2)
				if self.report: self.report.display(2)
				self.solutions.append(self.listValues())
				if self.report: self.report.display(5)
				if self.report and self.report.do(2):
					self.report.message("continuing to look for alternative solutions...",2)
				if stopAt and self.nsolutions >= stopAt:
					self.solving = False
					return self.nsolutions
			elif modified:
				#...the checkLoneValue function gave us new values
				#calling it again may yield further values
				return self.solve(stopAt)
			else:
				#...neither checkGroup nor checkLoneValues yielded any
				#new values, we will have to __guess...
				if self.report and self.report.do(3):
					self.report.message("stuck",3)
				randomCells = self.cells[:]
				random.shuffle(randomCells)
				alternatives = [len(cell.possibles) for cell in randomCells]
				try:
					#...for those cells which have not yet been allocated a value,
					#select one which has the smallest number of possible values...
					nalt = min( [p for p in alternatives if p > 1] )
					if self.report and self.report.do(2):
						self.report.message(("trying %d alternatives..." % (nalt,)),2)
					self.__alternatives += nalt
					theCell = randomCells[alternatives.index(nalt)]
					#...store the information required to return to this state 
					#later, select one of the alternative values, and carry on
					#looking for a solution
					self.__guess = Guess(theCell,self,self.__guess)
					if self.report and self.report.do(3):
						self.report.message("__guessing...",3)
					self.reportCell(self.__guess.cell, True)
					return self.solve(stopAt)
				except ValueError: pass
		except ImpossibleGridError: pass
		#the last __guess we made has been fully explored, 
		#now we need to look at alternatives
		if self.report and self.report.do(2):
			self.report.message("abandon this leaf - reverse last choice",2)
		while self.__guess:
			self.restore(self.__guess.storedGrid)
			if self.__guess.next():
				#look at alternative values for the last __guess"
				if self.report and self.report.do(3):
					self.report.message("__guessing...",3)
				self.reportCell(self.__guess.cell,  True)
				return self.solve(stopAt)
			else:
				#we have exhausted the alternative values - 
				#look at the previous __guess"
				if self.report and self.report.do(2):
					self.report.message("exhausted possibilities for"
					" this choice, returning to previous decision",2)
				self.__guess = self.__guess.parent
		#all possibilites have been examined"
		if self.report and self.report.do(1):
			self.report.message(("%d solutions found (%d alternatives considered)" % (self.nsolutions,self.__alternatives)),1)
		if self.nsolutions == 0:
			self.restoreValues(self.__restoreOnFail)
			if self.report and self.report.do(2):
				self.report.message("no solution found",2)
		self.solving = False
		return self.nsolutions

	def __eq__(self, other):
		if self.size != other.size: 
			return False
		for i in range(len(self.cells)):
			if self.cells[i].value != other.cells[i].value: 
				return False
		return True

	def __ne__(self, other):
		return not self.__eq__(other)

	def toArray(self):
		a = numarray.array([cell.value for cell in self.cells])
		return numarray.resize(a,(self.size*self.size,self.size*self.size))

	def fromArray(self,array):
		i = 0
		for c in self.cells:
			c.setValue(array[i], False)
			i += 1

	def transform(self, a):
		"""yields all possible variations on the given grid which can be obtained by swapping rows and columns
		the results are given as 'numarray' type arrays"""
		import transform
		matrix = a.toArray()
		product = numarray.matrixmultiply
		for m in (matrix, numarray.transpose(matrix)):
			for T1 in transform.Transformer.transformer(self.size):
				for T2 in transform.Transformer.transformer(self.size):
					yield product(product(T1,m),T2).flat

#**************************** class InputReader **************************"

class InputReader(object):
	"reads the initial grid data"

	def __init__(self, input=None, string='', size = 0, csvFile=None):
		self.__size = size
		self.__lineReader = None
		self.open(input, string, csvFile)

	#********************* properties ************************"

	def __getSize(self):
		if self.__size: return self.__size
		raise GridSizeError("grid size not set in InputReader")

	def __setSize(self, size):
		if self.__size: raise GridSizeError("attempt to reset grid size"
			" in report reader (already set to %d)" % size)
		self.__size = size

	size = property(__getSize, __setSize)

	#*********************** other methods *******************"

	def open(self, input = None, string = '', csvFile=None):
		if csvFile:
			self.__lineReader = CsvFileLineReader(csvFile)
			self.__size = self.__lineReader.size
		elif not input and not string:
			self.__lineReader = InteractiveLineReader(self.__size)
		else:
			if input:
				if input == '-': stream = sys.stdin
				else: stream = open(input)
				self.__lineReader = FileLineReader(stream)
			else: self.__lineReader = MemLineReader(string)
			self.__size = self.__lineReader.size
			
	def close(self):
		self.__lineReader.close()

	def dataLine(self):
		return self.__lineReader.dataLine()

#**************************** class FileLineReader **************************

class FileLineReader(object):
	"""reads starting grid data from a file in the format below

	['1020', '2010', '0240', '0000']
	or 
	1020201002400000
	uses the data to determine the size of the grid"""

	CHUNKS = 2
	SOLID = 1

	def __init__(self, stream):
		self.stream = stream
		self.length = 0
		self.data = []
		self.__type = None

	#********************* properties ************************

	def getSize(self):
		import math
		if not self.length:
			self.__lineBuffer = self.dataLine().next()
		return int(math.sqrt(self.length))
	size = property(getSize)

	#*********************** other methods *******************

	def dataSource(self): 
		data = self.stream.readline()
		return data.rstrip('\n')

	def dataLine(self):
		if not self.__type:
			data = self.dataSource()
			if not data: raise EOFError()
			if data[0] == '[':
				self.data = eval(data)
				self.__type = self.CHUNKS
			else: 
				self.data = data
				self.__type = self.SOLID
		l = len(self.data)
		if not self.length:
			if self.__type == self.CHUNKS and l in (4, 9, 16): self.length = l
			elif l in (16, 81, 256): self.length = int(math.sqrt(l))
			else: raise GridSizeError("grid data must form a square of"
								" size 4, 9 or 16")
		if self.__type == self.CHUNKS:
			for i in range(self.length):
				yield [self.data[i][j] for j in range(self.length)]
		else:
			for i in range(0,l,self.length):
				yield [self.data[i+j] for j in range(self.length)]

	def close(self):
		self.stream.close()


#**************************** class CsvFileLineReader **************************

class CsvFileLineReader(FileLineReader):
	"reads starting grid data from a file in csv format"
	def __init__(self, csvFile):
		self.file = csvFile
		FileLineReader.__init__(self, open(self.file))

	#********************* properties ************************

	def getSize(self):
		import math
		if not self.length: 
			self.dataLine().next()
			self.close()
			self.stream = open(self.file)
		return int(math.sqrt(self.length))
	size = property(getSize)

	#*********************** other methods *******************

	def dataSource(self): 
		data = self.stream.readline()
		return data.rstrip('\n')

	def dataLine(self):
		data = self.dataSource()
		if not data: raise EOFError()
		self.length = data.count(',')+1
		if self.length not in (4, 9, 16):
			raise GridSizeError("grid data must form a square of"
								" size 4, 9 or 16")
		for i in range(self.length):
			if i != 0: data = self.dataSource()
			yield [i.strip() and int(i) or 0 for i in data.split(',')]

#**************************** class MemLineReader **************************

class MemLineReader(FileLineReader):
	"reads grid data from a memory location"
	def dataSource(self): return self.stream
	def close(self): pass

#**************************** class InteractiveLineReader **************************

class InteractiveLineReader(object):
	"reads grid data from the keyboard"

	def __init__(self, size):
		self.length = size*size

	def dataLine(self):
		for i in range(self.length):
			line = raw_input("input values for row %d: " % (i+1))
			yield [eval(line[j]) for j in range(self.length)]
	
	def close(self):
		pass

#**************************** class GridSizeError **************************

class GridSizeError(Exception):

	def __init__(self, value):
		self.value = value

	def __str__(self):
		return repr(self.value)

#**************************** class ImpossibleGridError **************************

class ImpossibleGridError(Exception):

	def __init__(self, value):
		self.value = value

	def __str__(self):
		return repr(self.value)

#**************************** class StoredGrid **********************************

class StoredGrid(object):
	"holds the information needed to restore the state of the grid during solving"
	def __init__(self,grid):
		self.groups = [group.store() for group in grid.groups]
		self.cells = [cell.store() for cell in grid.cells]

#**************************** class Guess ***************************************

class Guess(object):
	"holds information about guesses made in finding a solution to the grid"
	def __init__(self, cell, grid, parent=None):
		self.cell = cell
		#remember the grid before this guess"
		self.storedGrid = StoredGrid(grid)
		#remember the earlier guesses weve already made"
		self.parent = parent
		self.nalt = len(self.cell.possibles)
		self.possibles = self.cell.possibles
		#take the 0th alternative"
		self.choice = 0
		self.cell.value = self.cell.possibles[0]
		self.report = grid.report

	def next(self):
		"""move the guess on to the next alternative

		return	True if there is a new alternative
				False if we have already exhausted the alternatives"""
		self.choice += 1
		if self.choice < self.nalt:
			if self.report and self.report.do(3):
				self.report.message(("choice = %d" % (self.choice,)),3)
			self.cell.value = self.possibles[self.choice]
			return True
		else:
			return False

#**************************** class Report ***************************************

class Report(object):
	"""supervises output to screen or file
	
	When the report object is created, it is given a list indicating what
	levels of messages it should print.  The list is determined according to
	some varibles in the grid for which it is called.  When a message arrives
	it is printed if its level is one to which the report should respond.
	There is also a function do() which can be called to indicate whether
	messages of a given level will be printed This can be used prevent
	unnecessary evaluation of the print expression in the calling function.
	
	The significance of the different print levels is as follows:
	
	0 always print to 'output' 
	1 always print to 'output' unless 'output' is
				diverted to '-'.  In that case print to stderr 
	2 print to 'output' unless 'quiet' option is set 
	3 print to 'output' if 'verbose' is set 
	4 print to 'output' if 'very verbose' is set 
	5 write to file if 'outputFile' is set 
	6 print to 'output' if 'debug' is set
	
	"""

	def __init__(self, grid, log=None, quiet=False, verbose=False, 
					outputFile=None):
		self.grid = grid
		self.outputFile = outputFile
		self.__log = None
		self.__quiet = False
		self.__verbose = False
		self.__printLevels = []
		#set values using properties"
		self.log = log
		self.quiet = quiet
		self.verbose = verbose
		self.linenum = 0

		self.on = True

	#******************* define properties *******************"

	def __setQuiet(self, on=True): 
		self.__quiet = on
		if on: self.__verbose = False
		self.__printLevels = []
	quiet = property(None, __setQuiet)
	def __setVerbose(self, on=True): 
		self.__verbose = on
		if on: self.__quiet = False
		self.__printLevels = []
	verbose = property(None, __setVerbose)

	def setLog(self, log):
		self.__log = log
	def getLog(self): return self.__log
	log = property(getLog, setLog)

	def __getLevel(self):
		if not self.__printLevels:
			self.__setLevel()
		return self.__printLevels
	printLevels = property(__getLevel)

	def __setLevel(self):
		"what levels of messages should we print to output stream"
		if (self.__quiet):
			self.__printLevels = [0, 1]
		elif(self.__verbose):
			self.__printLevels = [0, 1, 2, 3]
		else:
			self.__printLevels = [0, 1, 2]

	def checkLevel(self, level):
		if not self.on: return (False, None)
		if level == 5: 
			if self.outputFile: return (True, self.outputFile)
			else: return (False, None)
		if level == 1:
			if self.outputFile == sys.stdout and self.log == sys.stdout:
				return (True, sys.stderr)
			else:
				return (True, self.log)
		if not self.printLevels.count(level): return (False, None)
		else: return (True, self.log) 

	#****************** external methods **********************

	def setOptions(self, log=-999, quiet=-999, verbose=-999, outputFile=-999):
		if log != -999: self.log = log
		if quiet != -999: self.quiet = quiet
		if verbose != -999: self.verbose = verbose
		if ouputFile != -999: self.ouputFile = ouputFile

	def do(self, level):
		return self.checkLevel(level)[0]

	def message(self, message, level=0, newline=True, linenum=False, cells=None):
		(proceed, dest) = self.checkLevel(level)
		if not proceed: return
		if linenum: self.linenum += 1
		prefix = linenum and "[%d] " % self.linenum or ''
		if newline: print >> dest, prefix + message
		else: print >> dest, prefix + message,

#************************* class DisplayReport *************************************

class DisplayReport(Report):
	"""supervises output to screen or file. adds ability to display grid"""
	
	def __init__(self, grid, log=None, quiet=False, verbose=False, 
					csv=False, printGrid=False, outputFile=None, xml=False):
		Report.__init__(self, grid=grid, log=log, quiet=quiet, verbose=verbose, outputFile=outputFile)
		self.__printGrid = None
		self.__writeIt = None
		self.__printIt = self.grid.printIt
		#set values using properties"
		self.log = log
		self.printGrid=printGrid

	#******************* define properties *******************

	def __setPrintGrid(self, on=False):
		self.__printGrid = on
		self.__printLevels = []
	printGrid = property(None, __setPrintGrid)

	def __setXml(self, on=True):
		if on: self.__writeIt = self.grid.writeXml
		elif self.xml: self.__writeIt = self.grid.writeIt
	def __getXml(self): return self.__writeIt == self.grid.writeXml
	xml = property(__getXml, __setXml)

	def __setCsv(self, on=True):
		if on: self.__writeIt = self.grid.writeCsv
		elif self.csv: self.__writeIt = self.grid.writeIt
	def __getCsv(self): return self.__writeIt == self.grid.writeCsv
	csv = property(__getCsv, __setCsv)

	def setLog(self, log):
		if log is sys.stdout: self.__printIt = self.grid.printIt
		elif log: self.__printIt = self.__writeIt
		Report.setLog(self,log)
	log = property(Report.getLog, setLog)
	
	#****************** external methods **********************

	def setOptions(self, log=-999, quiet=-999, verbose=-999, 
						csv=-999, printGrid=-999, outputFile=-999):
		Report.setOptions(self, log=log, quiet=quiet, verbose=verbose, outputFile=outputFile)
		if csv != -999: self.csv = csv
		if printGrid != -999: self.printGrid = printGrid

	def display(self,level=0):
		if not self.on: return 
		(proceed, dest) = self.checkLevel(level)
		if not proceed: return
		if level == 5:
			dest.write(self.__writeIt())
		else:
			print >> dest, self.__printIt()

#************************** global functions ************************************
def createForwards(symmetry, grid=None, cursor=None, size=3):

	"""attempts to create a puzzle with the stipulated symmetry type
	works by adding cells in a symmetrical pattern to blank grid until
	a puzzle with a single solution is obtained"""
	import sudoSkeleton, random
	print "in create forwards- size = %d" % size
	if not grid: 
		grid = Grid(file='BLANK', cursor=cursor, size=size)
		grid.report.quiet = True
	elif cursor: grid.cursor = cursor
	grid.initializeSolveValues()
	skeleton = sudoSkeleton.Skeleton(size=3, type=symmetry)
	solution = None
	for i in range(300):
		if tryMore(grid, skeleton, solution): break
	else:
		print 'couldnt get a solution'
		return None
	if symmetry != 'N': return grid
	randomCells = grid.cells[:]
	random.shuffle(randomCells)
	for cell in randomCells:
		if cell.value and cell.isRedundant(): cell.value = 0
	return grid
	#return grid.savePuzzle(symmetry)

def tryMore(grid, skeleton, solution):
	"""adds cells to the given grid and tests to see whether the resulting puzzle is solvable
	uses the symmetry type defined by the given skeleton (the shape of the skeleton is not used), 
	and adds values which build up to the solution if one is provided -
	otherwise values are chosen randomly """
	
	import random
	before = StoredGrid(grid)
	b = grid.findZeros()
	chosen = skeleton.getCells(n=b[random.randrange(0,len(b))])
	for c in chosen:
		cell = grid.cells[c]
		if solution: cell.value = solution[c]
		else:
			try:
				l = len(cell.possibles)
				if l < 1: raise ImpossibleGridError("no posssibles in tryMore")
				v = cell.possibles[random.randrange(0,l)]
				cell.value = v
				for cell in grid.cells: cell.reducePossibles()
			except ImpossibleGridError:
				grid.restore(before)
				print "failed"
				return False
	if grid.count(silent=True) > 15:
		after = StoredGrid(grid)
		try:
			grid.solve(2)
			nsolutions = grid.nsolutions
		except ImpossibleGridError:
			nsolutions = 0
		if nsolutions >= 1:
			solution = grid.listValues()
			grid.restore(after)
			print "solved"
		else:
			grid.restore(before)
			print "failed"
			return False
		if nsolutions == 1:
			print "finished"
			if grid.report: grid.report.display()
			print len([cell.value for cell in grid.cells if cell.value])
			return True
		else: return False
	else: return False

#************************** main program ****************************************

def main(args):
	print args
	from optparse import OptionParser
	parser = OptionParser()
	parser.add_option("-v", "--verbose", 
						action="store_true", 
						dest="verbose", 
						default = False)
	parser.add_option("-V", "--very-verbose", 
						action="store_true", 
						dest="veryVerbose", 
						default = False)
	parser.add_option("-q", "--quiet", 
						action="store_true", 
						dest="quiet", 
						default = False, 
						help="Only print out statistics to stderr")
	parser.add_option("-z", "--grid-size", 
						action="store", 
						dest="size", 
						default = '3', 
						help="Number of cells in sub-grid 2, 3, or 4")
	parser.add_option("-H", "--human", 
						action="store_true", 
						dest="human", 
						default = False, 
						help="""Use the solution methods a 
						human being would use""")
	parser.add_option("-i", "--input", 
						action="store", 
						dest="inputFile", 
						help="""Read initial state from INPUTFILE, 
						or if INPUTFILE = - from sys.input 
						(no prompts will be given).  If this 
						option is not set, the user will be 
						prompted to enter data one row at a time""")
	parser.add_option("-x", "--xml-input", 
						action="store_true", 
						dest="xmlin", 
						default=False,
						help="""Indicates that the input
						file is in xml format.
						Only useful in conjunction with -i""")
	parser.add_option("-c", "--csv-input", 
						action="store_true", 
						dest="csvin", 
						default = False, 
						help="""Stipulates that the input file is in 
						'comma separated value' format
						Only useful in conjunction with -i""")
	parser.add_option("-n", "--count", 
						action="store_true", 
						dest="count", 
						default = False, 
						help="""Count the numbers of values and blanks 
						in the start grid (if -p is specified), 
						or the final grid (if -r is specified)""")
	parser.add_option("-p", "--print-only", 
						action="store_true", 
						dest="printOnly", 
						default = False, 
						help="""Only print out the start grid, 
						do not find a solution""")
	parser.add_option("-w", "--write-file", 
						action="store", 
						dest="writeFile", 
						help="Copy the start grid to WRITEFILE")
	parser.add_option("-o", "--output-file", 
						action="store", 
						dest="outputFile", 
						help="""Write the solutions to OUTPUTFILE, 
						one per line.  If OUTPUTFILE = - 
						the results are written to stdout.  
						Use of - implies -q""")
	parser.add_option("-X", "--xml-output", 
						action="store_true", 
						dest="xmlout", 
						default=False,
						help="""Indicates that the output
						file is required in xml format.
						Only useful in conjunction with -o or -w""")
	parser.add_option("-C", "--csv-output", 
						action="store_true", 
						dest="csvout", 
						default = False, 
						help="""Stipulates that the output should be in 
						'comma separated value' format
						Only useful in conjunction with -o, or -w""")
	parser.add_option("-l", "--log-to-file", 
						action="store", 
						dest="logFile", 
						help="""Write all output to LOGFILE 
						instead of the screen.  
						Grids will be written in file format, 
						not screen format.""")
	(options, args) = parser.parse_args()

	if options.xmlin:
		fileData = {'xmlFile': options.inputFile}
	elif options.csvin:
		fileData = {'csvFile': options.inputFile}
	else:
		fileData = {'file': options.inputFile}
	if options.human:
		from sudokuH import HGrid
		g = HGrid(size=eval(options.size), **fileData)
		g.solve = g.hsolve
	else:
		g = Grid(size=eval(options.size), **fileData)
	global G   # useful for debuging
	G = g

	g.report.verbose = options.verbose or options.veryVerbose
	g.report.printGrid = options.veryVerbose
	g.report.csv = options.csvout
	g.report.xml = options.xmlout
	g.report.quiet = options.quiet

	if options.writeFile: # write input data
		g.report.outputFile = open(options.writeFile,"w")
		g.report.display(5)
		g.report.outputFile.close()
		g.report.outputFile = None


	if options.printOnly:
		g.report.display()
		if options.count: g.count()
	else:
		f = None
		if options.outputFile == "-": 
			f = sys.stdout
			g.report.outputFile = f
			g.report.quiet = True
		elif options.outputFile: 
			f = open(options.outputFile,"w")
			g.report.outputFile = f
		if options.logFile:
			g.report.log = open(options.logFile,"w")

		g.solve()
		if options.outputFile: f.close()

if __name__ == "__main__":
	main(sys.argv)
