#! /usr/bin/env python

import sudokuH, sudokuWin, xmlReader
from sudokuTable import *
import wx, sys, wx.grid
from pictures import tick, cross

class SudokuBigTable(SudokuTable):
	"""Adds capability to display candidate values lists to SudukoTable.
	See comments for SudokuBigWindow"""
	userPossColour  = wx.Color(0,0,255)

	def __init__(self, parent, grid):
		self.bigFont = wx.Font(family=wx.ROMAN, style=wx.NORMAL, 
					weight=wx.NORMAL, pointSize=20, faceName="itc bookman")
		self.smallFont = wx.Font(family=wx.SWISS, style=wx.NORMAL, 
					weight=wx.NORMAL, pointSize=8, faceName="clean")
		self.showPossibles = False
		self.showUserPossibles = False
		self.possibles = [] #user possibles list
		SudokuTable.__init__(self, parent, grid=grid, cellSize=40)
		self.SetDefaultRenderer(wx.grid.GridCellAutoWrapStringRenderer())
		self.SetDefaultCellFont(self.bigFont)
		# we need to store the possibles list as well as the values
		SolutionStep.save = type(grid).store
		SolutionStep.restore = type(grid).restore

	def __initializePossibles(self):
		"creates new, empty self.possibles list"
		self.possibles = []
		for cell in range(self.size**4):
			self.possibles.append([])
		return self.possibles

	def readAllValues(self, showingSteps=False):
		"reads values from the underlying grid into the display"
		if not self.showPossibles and not self.showUserPossibles: 
			for i in range(self.size**2):
				for j in range(self.size**2):
					self.SetCellFont(i, j, self.bigFont)
					self.SetCellAlignment(i, j, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE)
			return SudokuTable.readAllValues(self, showingSteps)
		if self.showPossibles and not showingSteps:
			self.grid.initializeSolveValues(resetCounters=False)
		for i in range(self.size**2):
			for j in range(self.size**2):
				self.__readValue(i,j)
		self.Refresh()
	
	def __readValue(self, row=-1, col=-1, number=-1):
		"writes the cell's value to the display when possibles list is to be displayed"
		if number != -1: k = number
		else: k = row * self.size**2 + col
		
		if self.grid.cells[k].value:
			self.SetCellValue(row, col, str(self.grid.cells[k].value))
			self.SetCellFont(row, col, self.bigFont)
			self.SetCellAlignment(row, col, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE)
			return

		if self.showPossibles: 
			self.SetCellTextColour(row, col, self.normal)
			possibles = self.grid.cells[k].possibles
		else:  # showUserPossibles
			if not self.possibles: self.__initializePossibles()
			self.SetCellTextColour(row, col, self.userPossColour)
			possibles = self.possibles[k]
		string = ' '.join([str(i) for i in possibles])
		self.SetCellValue(row, col, string)
		self.SetCellFont(row, col, self.smallFont)
		self.SetCellAlignment(row, col, wx.ALIGN_LEFT, wx.ALIGN_TOP)

	def __updateCell(self, row=-1, col=-1, number=-1):
		"updates the cell showing the amended (auto) possibles list"
		if row == -1:
			row, col = number//self.size**2, number%self.size**2
		self.SetCellTextColour(row, col, self.normal)
		self.__readValue(row, col)
		self.Update()
		self.Refresh()

	def valueChosen(self, event):
		"sets the value chosen by the user, updates possibles lists for affected cells"
		row, col = self.clickRow, self.clickCol
		if not self.showPossibles: 
			self.SetCellFont(row, col, self.bigFont)
			self.SetCellAlignment(row, col, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE)
			return SudokuTable.valueChosen(self, event)
		import sets
		value = event.GetId()
		i = self.size*self.size*row + col
		self.grid.cells[i].value = value
		self.BeginBatch()
		self.__updateCell(row, col)
		self.grid.initializeSolveValues()
		for j in sets.ImmutableSet([c.number for s in self.grid.cells[i].groups for c in s.cells if c.number is not i]):
			self.__updateCell(number=j)
		self.EndBatch()
		self.undoBuffer.store()

	def editPossibles(self, row, col):
		"""brings up a mini frame for the user to specify candidate list
		When the user has hit the ok button, the possiblesModified method is called"""
		#save values identifying cell, for use by possiblesModified method
		self.clickRow, self.clickCol = row, col
		self.SetGridCursor(row,col)
		PossiblesDialog.getValues(self.parent, 
									self.possiblesModified, # callback function
									self.possibles[row*self.size**2+col], 
									wx.GetMousePosition())

	def possiblesModified(self, possibles):
		"updates the possibles list for a cell with the user choices"
		row, col = self.clickRow, self.clickCol
		self.possibles[row*self.size**2+col] = possibles[:]
		self.__updateCell(row,col)
		self.undoBuffer.store()

	def newFile(self):
		"actions to be carried out when a new file is read"
		if isinstance(self.grid.reader,xmlReader.XmlReader):
			self.possibles = self.grid.reader.possibles
		else:
			self.__initializePossibles()
	
	def storeUndo(self):
		"store values in undo buffer - include user possibles"
		possibles = []
		if self.possibles: 
			for cp in self.possibles: possibles.append(cp[:])
		return (self.grid.listValues(), self.clickRestore, possibles)

	def restoreUndo(self, data):
		"restore values if user selects undo - include user possibles"
		self.grid.restoreValues(data[0])
		if self.redundantsHighlighted: 
			self.highlightRedundants()
		self.clickRestore = data[1]
		if data[2]:
			self.possibles = []
			for cp in data[2]: self.possibles.append(cp[:])
		self.readAllValues()
		
#****************************** class SudokuBigWindow *********************************

class SudokuBigWindow(sudokuWin.SudokuWindow):
	"""Adds capability to display candidate values lists to SudukoWindow.
	There are two alternative lists - an automatically generated one which
	is used internally for finding the solution and which the user may view;
	and an optional list maintained by the user.  This class adds extra menu
	items to deal with these lists, and uses a different Table class from its
	parent.
	Within the program the lists of candidate values are called possibles.  But
	the more correct term 'candidates' is used in communicating with the user.
	Both terms are used interchangeably in the comments."""

	def __init__(self, *args, **kwds):
		sudokuWin.SudokuWindow.__init__(self, *args, **kwds)
		self.SetSize((600,720))
		s = self.table.grid.size**2 * 40 + 20
		self.table.SetSizeHints(s,s)
		self.printer = Printer(self)

	def populate(self):
		sudokuWin.SudokuWindow.populate(self)
		self.mPossibles = self.settingsMenu.AppendCheckItem(-1, "show &candidates list", 
				"candidates lists will be maintained automatically for each cell").GetId()
		self.mUserPossibles = self.settingsMenu.AppendCheckItem(-1, 
				"show &user candidates list", 
				"allows the user to maintain his or her lists of candidate values").GetId()
		wx.EVT_MENU(self, self.mPossibles, self.showPossibles)
		wx.EVT_MENU(self, self.mUserPossibles, self.showUserPossibles)
	
	def showPossibles(self, event):
		"called from menu"
		if self.settingsMenu.IsChecked(self.mPossibles):
			#turn show (auto) possibles on
			self.settingsMenu.Check(self.mUserPossibles, False)
			self.table.showPossibles = True
			self.table.showUserPossibles = False
		else:
			#turn show (auto) possibles off
			self.table.showPossibles = False
		self.table.readAllValues()
	
	def showUserPossibles(self, event):
		"called from menu"
		if self.settingsMenu.IsChecked(self.mUserPossibles):
			# turn show user candidates on
			self.settingsMenu.Check(self.mPossibles, False)
			self.table.showPossibles = False
			self.table.showUserPossibles = True
			self.gridMenu.Enable(self.mChangePossibles,True)
			self.__doMenuChangePossibles(True)
		else:
			# turn show user candidates off
			self.table.showUserPossibles = False
			if self.gridMenu.IsChecked(self.mChangePossibles):
				self.__doMenuChangePossibles(False)
			self.gridMenu.Enable(self.mChangePossibles,False)
		self.table.readAllValues()
	
	def __doMenuChangePossibles(self, on):
		event = wx.CommandEvent(wx.wxEVT_COMMAND_MENU_SELECTED, self.mChangePossibles)
		event.SetInt(on and 1 or 0)
		self.GetEventHandler().ProcessEvent(event)

	def createTable(self, grid):
		"this method is called by SudukoTable.__init__ it specifes the type of table"
		return SudokuBigTable(self, grid=grid)

	def writeXml(self):
		return str(xmlReader.XmlWriter(self.table.grid, self.table.possibles))

	def font(self, event):
		"called from the menu when font is selected"
		dialog = wx.FontDialog(self,wx.FontData())
		ok = dialog.ShowModal() == wx.ID_OK
		if not ok: return
		font = dialog.GetFontData().GetChosenFont()
		self.table.bigFont = font

#****************************** class Printer *******************************

class Printer(sudokuWin.Printer):

	def OnPrintPage(self, n):
		sudokuWin.Printer.OnPrintPage(self,n)
		if not self.parent.table.showPossibles and not self.parent.table.showUserPossibles: return

		# print possibles lists

		z=self.grid.size
		
		(stepx, stepy) = self.lengthx/z/z, self.lengthy/z/z
		innerMargin = stepx/10
		startx = self.marginx + self.lengthx/z/z/2 - stepx /2 + innerMargin
		starty = self.marginy + self.lengthy/z/z/2 - stepy /2 + innerMargin
		maxLength = stepx - 2*innerMargin
		dc = self.GetDC()
		dc.SetFont(wx.Font(12, wx.FONTFAMILY_SWISS, 
				wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))

		for y in range(z*z):
			for x in range(z*z):
				k = y*z*z+x
				if self.parent.table.showPossibles:
					posslist = self.grid.cells[k].possibles
				else: posslist = self.parent.table.possibles[k]
				if self.grid.cells[k].value: continue
				text = ', '.join([str(p) for p in posslist])
				charsToPrint = len(text)
				charsPrinted = 0
				offset = 0
				extents = dc.GetPartialTextExtents(text)
				while charsPrinted < charsToPrint:
					# new line of text starts here
					# loop over the extents array looking for the 1st comma after the end of line
					for i in range(1,charsToPrint-charsPrinted+1,3):
						if i > charsToPrint-charsPrinted-1: i = charsToPrint-charsPrinted-1
						if extents[i] > maxLength:
							if i==1: #we cant fit a single character but try anyway
								pass
							else:
								# go back to previous comma
								i = lasti+1
							dc.DrawText(text[:i], startx+x*stepx, starty+y*stepy+offset)
							(w, h, d, l) = dc.GetFullTextExtent(text[i+1:])
							text = text[i+1:]
							extents = dc.GetPartialTextExtents(text)
							offset += h + d + l
							charsPrinted += i + 1 # add an extra 1 to skip the space
							
							break
						lasti = i
					else:
							dc.DrawText(text, startx+x*stepx, starty+y*stepy+offset)
							charsPrinted = charsToPrint

#************************** class PossiblesDialog *******************************

class PossiblesDialog(wx.MiniFrame):
	"""mini window for user to specifiy candidate values for a cell
	Only one instance is meant to be instantiated.  
	Only the getValues class method should be called by other classes.
	That method specifies a callback function to which a list of the candidate
	values is sent when the dialog is closed"""

	dialog = None	
	len = 25

	def __init__(self, parent):
		self.parent = parent
		self.cancelled = False
		self.callback = None
		wx.MiniFrame.__init__(self, parent, -1, "cand", 
				size=(self.len+5,self.len*(parent.table.grid.size**2+2)+20), 
				style=wx.FRAME_FLOAT_ON_PARENT|wx.CAPTION)
		self.box = wx.BoxSizer(wx.VERTICAL)
		self.buttons = {}
		for i in range(1,parent.table.grid.size**2+1):
			tb = wx.ToggleButton(self,-1,"&%d" % i)
			tb.SetSizeHints(self.len,self.len,self.len,self.len)
			self.buttons[i] = tb
			self.box.Add(tb)
		okPic = wx.BitmapFromXPMData(tick)
		okButton = wx.BitmapButton(self,-1,okPic)
		okButton.SetSizeHints(self.len,self.len,self.len,self.len)
		self.box.Add(okButton)
		cancelPic = wx.BitmapFromXPMData(cross)
		cancelButton = wx.BitmapButton(self,-1,cancelPic)
		cancelButton.SetSizeHints(self.len,self.len,self.len,self.len)
		self.box.Add(cancelButton)
		self.SetSizer(self.box)
		wx.EVT_BUTTON(self, okButton.GetId(), self.__ok)
		wx.EVT_BUTTON(self, cancelButton.GetId(), self.__cancel)
	
	def getValues(cls, parent, callback, oldPossibles, position):
		"shows the control at the specified position" 
		if not cls.dialog: cls.dialog = cls(parent)
		cls.dialog.callback = callback
		cls.dialog.__setPossibles(oldPossibles)
		cls.dialog.Move(position)
		cls.dialog.Show()
	getValues = classmethod(getValues)

	def __getPossibles(self):
		"returns a list of the possibles values set in the dialog"
		return [i for i in range(1,self.parent.table.grid.size**2+1) 
				if self.buttons[i].GetValue()]
	
	def __setPossibles(self, list):
		"sets the dialog buttons to correspond to the given list"
		for i in self.buttons.keys(): self.buttons[i].SetValue(i in list)

	def __ok(self, event):
		self.cancelled = False
		self.Hide()
		self.callback(self.__getPossibles())
	
	def __cancel(self, event):
		self.cancelled = True
		self.Hide()

#****************************** main program *********************************

def main(*args, **kwds):
	(options, args) = sudokuWin.parseOptions()
	
	sudokuWin.GuiApp((options,  args), windowClass=SudokuBigWindow)
	return

if __name__=="__main__":
	main(sys.argv)
