#! /usr/bin/env python

"classes needed to solve a sudoku grid in the same way that a human would"

import sys, sets
from sudoku import Grid, Cell, Group, Report

#**************************** class HCell *************************************

class HCell(Cell):
	"Cell with extra data and methods to deal with fragments"

	def __init__(self, groups, grid):
		Cell.__init__(self, groups, grid)
		self.fragments = []
	
	def fromFragments(self):
		"sets the Cell data to conform the the information obtained from fragments"
		if self.value or len(self.possibles) == 1: return (False, False)
		possiblesModified = False
		#remove from the cell possibles list any values not in the both fragment possibles lists
		possibles = sets.Set(self.fragments[0].possibles)
		possibles.intersection_update(self.fragments[1].possibles)
		for p in self.possibles:
			if p not in possibles:
				try:
					self.possibles.remove(p)
					possiblesModified = True
				except ValueError: pass

		# if theres only one possibility left, set it
		valueSet = len(self.possibles) == 1
		if valueSet: self.value = self.possibles[0]
				
		return (valueSet, possiblesModified)

	def setValue(self, value, update=True):
		Cell.setValue(self, value, update)
		for s in self.fragments:
			if s.requires and value in s.requires: s.requires.remove(value)
		for s in self.fragments: s.value = value
		return True
	value = property(Cell._Cell__getValue, setValue)

	#********************** class methods *******************"
	
	def alreadyGot(cls, list, value): 
		"""returns whether the cells in the list contain the value"""
		
		return value in [c.value for c in list]
	alreadyGot = classmethod(alreadyGot)

#************************* class Fragment *************************************

class Fragment(object):
	"""hold info about a set of size cells within the same row and the same box, or within the same col and the same box"""

	def __init__(self, controller, cells, groups, number):
		self.cells = cells # size cells
		self.groups = groups # 2 groups
		self.values = [] # size values
		self.requires = None # up to size values
		self.satisfied = False
		self.number = number
		self.controller = controller
		self.initializeSolveValues()
		self.__size = len(cells)
	
	def initializeSolveValues(self):
		self.values = []
		self.requires = None

	def __str__(self):
		typ = self.number < self.__size**3 and "row" or "column"
		return "%s fragment %s to %s" % (typ, self.cells[0], self.cells[self.__size-1])
	
	def alreadyGot(cls, list, value):
		"returns whether the fragments in the list have the value"
		currentValues = []
		for x in list: currentValues.extend(x.values)
		if value in currentValues: return True
		else: return False
	alreadyGot = classmethod(alreadyGot)

	def fromCells(self):
		modified = valueSet = False
		if self.satisfied: return (False, False)
		for c in self.cells: 
			if c.value: 
				self.values.append(c.value)
				modified = True
		# if there are only size possible values then these must be the values
		if (len(self.possibles) == self.__size) and not self.values: 
			self.values = self.possibles
			valueSet = True
		self.__checkRequires()
			
		return (valueSet, modified)

	def __getPossibles(self):
		"reduce the fragments possibles list by examining the cells' lists"
		# build a local possibles list holding the possibles from all the cells in the fragment
		possibles = sets.Set()
		for c in self.cells: 
			for p in c.possibles:
				possibles.add(p)
		return list(possibles)
	possibles = property(__getPossibles)

	
	def __checkRequires(self):
		if len(self.values) == self.__size and not self.requires:
			self.requires = self.values[:]
		if self.requires:
			for c in self.cells:
				if c.value:
					try: self.requires.remove(c.value)
					except ValueError: pass
					if c.value not in self.values: self.values.append(c.value)
	
	def setValue(self, v):
		if v in self.values: return
		self.values.append(v)
		self.__checkRequires()
		modified = False
		for s in self.groups: 
			#s.removeReqs(v)
			for c in s.cells:
				if c in self.cells: continue
				try: 
					c.possibles.remove(v)
					modified = True
				except ValueError: pass
		return modified
	def getValue(self): return self.values
	value = property(getValue, setValue)
	
#**************************** class FragmentController *******************************

class FragmentController(object):
	"""looks after the fragments

	the start values for the grid are set by startVals(), data is kept up to date
	by the setValue methods of HCell and Fragment"""

	def __init__(self, grid):
		self.grid = grid
		self.fragments = []
		self.setStandingData(grid)
		self.on = False # should be explicitly turned on when needed
	
	def initializeSolveValues(self):
		for s in self.fragments: s.initializeSolveValues()
		self.on = False # wait to be turned on

	def setStandingData(self, grid):
		fragmentsForCell = {}
		i = 0
		print "standing data - size = %d" % self.grid.size
		sz = self.grid.size
		for r in range(sz*sz):
			for c in range (0,sz*sz,sz):
				cells = tuple([grid.cells[j] for j in range(r*sz*sz+c,r*sz*sz+c+sz)])
				groups = (grid.groups[r], grid.groups[2*sz*sz+(r/sz)*sz+(c/sz)])
				fragment = Fragment(self, cells, groups, i)
				for s in groups: s.fragments.append(fragment)
				self.fragments.append(fragment)
				i += 1
				for cell in cells: fragmentsForCell[cell] = fragment
		for r in range(0,sz*sz,sz):
			for c in range(sz*sz):
				cells = tuple([grid.cells[j] for j in range(r*sz*sz+c,(r+sz)*sz*sz+c,sz*sz)] )
				groups = (grid.groups[c+sz*sz], grid.groups[2*sz*sz+sz*(r/sz)+(c/sz)])
				fragment = Fragment(self, cells, groups,i)
				for s in groups: s.fragments.append(fragment)
				self.fragments.append(fragment)
				i += 1
				for cell in cells: 
					rowFragment = fragmentsForCell[cell]
					fragmentsForCell[cell] = (rowFragment, fragment)
		for c in grid.cells:
			c.fragments = tuple(fragmentsForCell[c])

	def startVals(self):
		modified = False
		for s in self.fragments: 
			(val, mod) = s.fromCells()
			if val or mod: modified = True
		for c in self.grid.cells: 
			(val, mod) = c.fromFragments()
			if val or mod: modified = True
		return modified

	def nPossibles(self, sgroups, n=2):
		"""ref: http://www.simes.clara.co.uk/programs/sudokutechnique9.htm
		reducing candidates rule"""
		for s in sgroups:
			if not s.values: continue
			# check that the number of blank cells is greater than n
			blanks = [bc for bc in s.cells if not bc.value]
			l = len(blanks)
			if l <= n: continue
			# get all possible combinations of n unset cells
			for combo in permute(0,l,n):
				possibles = sets.Set(s.values)
				# remove from the list the possibles values for other cells in the group
				for i in range(l):
					if i in combo: continue
					possibles.difference_update(blanks[i].possibles)
				# if the number of remaining possibles is the same as the number of cells we're combining
				if len(possibles) == n:
					#remove all values but those in possibles from the possibles list of the cells in the combination
					valueRemoved = False
					for cell in [blanks[c] for c in combo]:
						for i in cell.possibles:
							if i not in possibles:
								valueRemoved = True
								cell.possibles.remove(i)
					if valueRemoved:
						self.grid.report.message("the values %s in %s can only be "
								"provided by %s" % (' and '.join([str(j) for j in possibles]), 
								s, ' or '.join([str(blanks[i]) for i in combo]))) 
					return valueRemoved
		return False

#**************************** class HGroup *************************************

class HGroup(Group):
	"Group with extra data to deal with fragments"

	def __init__(self, cells=None, grid=None):
		Group.__init__(self, cells, grid)
		self.fragments = []
		
#**************************** class PairsList ****************************************

class PairsList(object):
	"stores and gives information about the list of conjugate pairs for the grid"
	def __init__(self, size):
		self.__byValByCell = {}
		self.__byValByGroup = {}
		self.__size = size

	def add(self, pair):
		if not self.__byValByCell.has_key(pair.value):
			self.__byValByCell[pair.value] = {}
		for cell in pair.cells:
			if not self.__byValByCell[pair.value].has_key(cell):
				self.__byValByCell[pair.value][cell] = []
			self.__byValByCell[pair.value][cell].append(pair)
		if not self.__byValByGroup.has_key(pair.value):
			self.__byValByGroup[pair.value] = {}
		if not self.__byValByGroup[pair.value].has_key(pair.group):
			self.__byValByGroup[pair.value][pair.group] = []
		self.__byValByGroup[pair.value][pair.group].append(pair)
	
	def forVal(self,value):
		"generates all the pairs for this value"
		if not self.__byValByGroup.has_key(value): return
		for i in self.__byValByGroup[value].keys():
			#return [p for p in self.__byValByGroup[value][i]]
			for p in self.__byValByGroup[value][i]:
				yield p

	def has_value(self,value):
		"returns whether there are any pairs for this value"
		return self.__byValByGroup.has_key(value)

	def has_cell(self, value, cell):
		"returns whether there are any pairs containing this cell for this value"
		return self.__byValByCell[value].has_key(cell) 

	def forCell(self, value, cell):
		"returns a list of all the pairs for the given value containing this cell"
		return self.__byValByCell[value][cell]
	
	def dump(self):
		for i in range(1,self.__size*self.__size+1):
			for x in self.forVal(i):
				print x

#**************************** class Pairs ****************************************

class Pairs(object):

	def __init__(self, value, cells, group, pairsList):
		c = list(cells)
		c.sort()
		self.cells = c
		self.group = group
		self.value = value
		pairsList.add(self)
	
	def __str__(self):
		return "group = %s\ncells = %s\nvalue = %d" % (self.group, ','.join([str(c) for c in self.cells]), self.value)

	def otherEnd(self, c):
		return c is self.cells[0] and self.cells[1] or self.cells[0]

#***************************** class HGrid *****************************

class HGrid(Grid):
	def __init__(self, size=3, file=None, string='', id=0, cursor=None, table=None, grid=None):
		"specify one of size, file, string or grid"
		if grid: string = grid.writeIt()
		Grid.__init__(self, size=size, file=file, string=string)
		self.statistics = Statistics()
		"variables used in finding a solution using human methods"
		self.pairsList = None
		self.xWingsFound = sets.Set()
		self.fragmentController = None

	def __ac(list, start, end):
		newlist = []
		s = len(list)
		list.append('')
		for i in range(start,end):
			list[s] = i
			newlist.append(list[:])
		return newlist
	__ac = staticmethod(__ac)

	def permute(start, end, n):
		"makes a list of all possible permutations of n elements in the range"
		list = HGrid.__ac([],start,end)
		newlist = []
		for i in range(n-1):
			for el in list:
				newlist.extend(HGrid.__ac(el,el[i]+1,end))
			if i < n-2:
				list = newlist[:]
				newlist = []
		return newlist
	permute = staticmethod(permute)

	def initializeSolveValues(self, resetCounters=True):
		Grid.initializeSolveValues(self, resetCounters)
		if self.fragmentController: self.fragmentController.initializeSolveValues()
		self.pairsList = None
		self.xWingsFound = sets.Set()

	def specifyCell():
		"specifies the class to be used for cells"
		return HCell
	specifyCell = staticmethod(specifyCell)

	def specifyGroup():
		"specifies the class to be used for groups"
		return HGroup
	specifyGroup = staticmethod(specifyGroup)
	

	def reportCell(self, cell, dummy=False):
		for c in self.cells: c.reducePossibles()
		if not self.report: return
		self.report.display(4)
		if self.report.do(3):
			self.report.message("%s set to %d %s" % 
				(cell, cell.value, "(no other values were possible)"),
				3, 
				linenum=True,
				cells=((cell.row,cell.col),))

	def reportCellInGroup(self, cell, group):
		"""reports that a value has been asigned to a cell or fragment
		
		because no other cell could provide that value for a group.  the actual
		output is controlled by instance variables verbose and printGrid"""

		if type(cell) is not HCell:
			self.statistics.inc("fragments")
		if not self.report: return
		self.report.display(4)
		if self.report.do(3):
			for c in self.cells: c.reducePossibles()
			if type(cell) is HCell:
				self.report.message( "%s set to %s "
					"(no other cell in %s could have that value)" % 
					(cell, cell.value, group),3,linenum=True, cells=((cell.row,cell.col),))
			else:
				# here cell is in fact a row or column fragment
				v = cell.value
				v = v[len(v)-1]
				self.report.message( "a cell in %s must have the value %s "
					"(no other fragment in %s could have that value)" % 
					(cell, v, group),
					3,
					linenum=True,
					cells = [(c.row,c.col) for c in cell.cells]
					)

	def hsolve(self):
		"""finds a solution for the grid, using tactics a human solver could use

		the amount of detail in the report is controlled by Report object"""
		if not self.solving:
			self.initializeSolveValues()
			self.solving = True
			self.restoreOnFail = self.listValues()
			self.statistics = Statistics()
		"""go through the cells, for each one eliminating the values of other cells 
		in each of the groups of which it is part"""
		modified = [cell.checkLoneValue() for cell in self.cells].count(-1) 

		""" for each group go through the list of values not yet used and check 
		how many of the cells which comprise the group could take that value - 
		if there is only one then set the value of the cell"""
		if self.checkGroup(self.groups): return self.hsolve()
		if self.fragmentController and self.fragmentController.on:
			# check the cell values within the fragments
			if self.checkGroup(self.fragmentController.fragments): return self.hsolve()
			# check the fragment values within the groups
			if self.checkGroup(self.groups[:2*self.size*self.size], "fragments"): return self.hsolve()
			#for box groups check the row and column fragments separately
			if self.checkGroup(self.groups[2*self.size*self.size:], "fragments[:self.size]"): return self.hsolve()
			if self.checkGroup(self.groups[2*self.size*self.size:], "fragments[self.size:]"): return self.hsolve()
		if [cell.value for cell in self.cells].count(0) == 0:
			"...we have found a solution"
			self.nsolutions += 1
			if self.report.do(2):
				self.report.message("solved",2)
			self.report.display(2)
			self.solutions.append(self.listValues())
			self.report.display(5)
			self.solving = False
			return True
		elif modified:
			#...the checkLoneValue function gave us new values
			#calling it again may yield further values
			return self.hsolve()
		else:
			#...neither checkGroup nor checkLoneValues 
			#yielded any new values, we will have to try other strategies...

			for n in range(2,8):
				if self.nMatch(n):
					self.statistics.inc("ngroups", n)
					return self.hsolve()
			
			for n in range(2,8):
				if self.nPossibles(n):
					self.statistics.inc("npossibles", n)
					return self.hsolve()

			if not self.fragmentController:
				self.fragmentController = FragmentController(self)
				self.fragmentController.on = False
			if not self.fragmentController.on:
				self.fragmentController.on = True
				if self.fragmentController.startVals():
					return self.hsolve()

			if self.conjugatePairs():
				for n in range(2,self.size*self.size):
					if self.xWing(n):
						self.statistics.inc("xwings")
						return self.hsolve()
				if self.CPChain():
					self.statistics.inc("chains")
					return self.hsolve()

			if self.report.do(3):
				self.report.message("stuck",3)
				self.report.display()
		#all possibilites have been examined
		if self.nsolutions == 0:
			self.statistics.impossible = True
			self.restoreValues(self.restoreOnFail)
			if self.report.do(4):
				self.report.display(4)
			if self.report.do(2):
				self.report.message("no solution found",2)
		self.solving = False
		return False

	def tightLooseLink(self, value, start, tight, close, cellsInLoop=None, modified=False, maxCPs=0):
		if maxCPs == 0: maxCPs = self.size*self.size
		k = value
		if not cellsInLoop: cellsInLoop = []
		#start here for another link in the chain
		a = start is tight.cells[0] and tight.cells[1] or tight.cells[0]
		inLoose = sets.Set([c for s in a.groups for c in s.cells if c is not a and c is not start and s is not tight.group])
		#possB = inLoose.copy()
		possB = list(inLoose)
		for b in inLoose:
			if not self.pairsList.has_cell(value=k,cell=b): 
				possB.remove(b)
				continue
			pairsForB = [p.cells for p in self.pairsList.forCell(value=k,cell=b)]
			for c in close:
				if c is a: continue
				lookup = b < c and list((b,c)) or list((c,b))
				if lookup in pairsForB:
					# found an x-wing - but make sure we dont already have it
					new = sets.Set((start, a, b, c))
					new = new.union([x[0] for x in cellsInLoop] + [x[1] for x in cellsInLoop]).__as_immutable__()
					if (k, new) not in self.xWingsFound:
						self.xWingsFound.add((k, new))
						#remove k values from the possibles lists of cells in the groups which contain a and b and those which contain start and c
						#self.report.message("xwing: start = %s, a=%s, b=%s, c=%s" % (start, a, b, c))
						messages = []
						for s in self.groups:
							providers = ()
							if (start in s.cells and c in s.cells): providers = (start, c)
							elif (a in s.cells and b in s.cells): providers = (a, b)
							if providers:
								#self.report.message("removing %d values from %s" % (k, s))
								printThis = False
								for cell in s.cells:
									if cell is a or cell is b or cell is c or cell is start: continue
									try:
										cell.possibles.remove(k)
										modified = True
										#self.report.message("%d removed from %s possibles" % (k, cell))
										printThis = True
									except ValueError: pass
								if printThis:
									messages.append("the value %d in %s must be provided by %s" % 
										(k, s, ' or '.join([str(p) for p in providers])))
						if messages:
							self.report.message('\nand '.join(messages) + \
								 ".\nThis is because there is a chain of cells which could"\
								 " have the value %d which relate to each other as follows:"\
								 "\nIf %s is not %d then %s must be, if %s is %d then %s cannot"\
								 " be, if %s is not %d then %s is %d. We could follow the "\
								 "chain backwards to prove that if %s is not %d then %s is."\
								 " Similarly if %s is not %d then %s is and if %s is not %d "\
								 "then %s is." % 
								 (k, start, k, a, a, k, b, b, k, c, k, c, k, 
								 start, b, k, a, a, k, b), 
								 2, 
								 linenum=True,
								 cells=[(x.row,x.col) for x in (start, a, b, c)])
							self.statistics.inc("xwings")
						#self.report.message( "found an x wing: value = %d - cells are (%s, %s, " % \
							#(k, start, a), 2, False, linenum=True)
						#for i in cellsInLoop:
							#self.report.message( "%s, %s, " % (i[0], i[1]), 2, False)
						#self.report.message( "%s, %s)" % (b, c), 2)
							
		if not modified and possB: # couldnt close this time - but can try longer chain
			if len(cellsInLoop) > maxCPs -self.size: return False # set max chain length
			cellsInLoop.append((start,a))
			for b in possB:
				for p in self.pairsList.forCell(value=k,cell=b):
					return self.tightLooseLink(value=k, start=b, tight=p, close=close, cellsInLoop=cellsInLoop, modified=modified, maxCPs = maxCPs)
		return modified

	def conjugatePairs(self):
		"find all the conjugate pairs"

		self.pairsList = None
		for s in self.groups:
			for val in s.requires:
				providers = [c for c in s.cells if val in c.possibles]
				if len(providers) == 2:
					if not self.pairsList: self.pairsList = PairsList(self.size)
					Pairs(val, providers, s, self.pairsList)
		return self.pairsList and True or False

	def CPChain(self):
		if not self.pairsList: return False

		usedAChain = False
		newA = sets.Set()
		newB = sets.Set()
		for k in range(1,self.size*self.size+1):
			#loop over the pairs
			if not self.pairsList.has_value(k): continue
			allCells = sets.Set()
			for p in self.pairsList.forVal(k):
				for i in range(2):
					allCells.add(p.cells[i])
			while True:
				try:
					c = allCells.pop()
				except KeyError: break
				p = self.pairsList.forCell(value=k, cell=c)[0]
				newA.clear()
				newB.clear()
				newA.add(p.cells[0])
				newB.add(p.cells[1])
				for i in range(2):
					allCells.discard(p.cells[i])
				modified = True
				while modified:
					modified = False
					for c in newA:
						for q in self.pairsList.forCell(value=k, cell=c):
							d = q.otherEnd(c)
							l = len(newB)
							newB.add(d)
							if len(newB) > l: modified = True
							allCells.discard(d)
					for c in newB:
						for q in self.pairsList.forCell(value=k, cell=c):
							d = q.otherEnd(c)
							l = len(newA)
							newA.add(d)
							if len(newA) > l: modified = True
							allCells.discard(d)
				#we have our chain
				#take all possible combinations of cells from each list
				for a in newA:
					for b in newB:
						for s in self.groups:
							valueRemoved = False
							if a in s.cells and b in s.cells:
								for c in s.cells:
									if c is a or c is b: continue
									try: 
										c.possibles.remove(k) 
										valueRemoved = True
									except ValueError: 
										pass
							if valueRemoved:
								usedAChain = True
								self.report.message( "CP chain for value %d linking %s and %s in %s" % (k, a, b, s), 2, linenum=True)
		return usedAChain

	def xWing(self, n):
		"ref: http://www.angusj.com/sudoku/hints.php"
			
		if not self.pairsList: return False
		modified = False

		for k in range(1,self.size*self.size+1):
			#loop over the pairs
			if not self.pairsList.has_value(k): continue
			for tight in self.pairsList.forVal(k):
				# try another starting pair
				closing = tight.cells[0]
				closingMatches = [c for s in closing.groups for c in s.cells if c != closing]
				if self.tightLooseLink(value=k, start=closing, tight=tight, close=closingMatches, maxCPs = n):
					modified = True
		return modified
		
	def nMatch(self, n):
		for s in self.groups:
			# check that the number of blank cells s greater than n
			if len([bc for bc in s.cells if not bc.value]) <= n:
				continue
			ritSiz = [c for c in s.cells if len(c.possibles) == n]
			ritSiz.extend([c for c in s.cells if len(c.possibles) in range(2,n)])
			l = len(ritSiz)
			for i in range(0, l-n+1):
				if len(ritSiz[i].possibles) < n: break
				cellsMatched = [ritSiz[i]]
				for j in range(i+1, l):
					if sets.Set(ritSiz[j].possibles).issubset(sets.Set(ritSiz[i].possibles)):
						cellsMatched.append(ritSiz[j])
						if len(cellsMatched) >= n:
							valueRemoved = False
							for cell in s.cells:
								if cell in cellsMatched: continue
								for val in ritSiz[i].possibles:
									try: 
										cell.possibles.remove(val) 
										valueRemoved = True
									except ValueError: 
										pass
							if valueRemoved:
								message = "%s in %s can hold only the values %s so " \
										"they must have one of those values each" % (
										' and '.join([str(m) for m in cellsMatched]), 
										s, 
										' or '.join([str(p) for p in ritSiz[i].possibles]))
								self.report.message(message, 2, linenum=True, 
										cells=([(m.row,m.col) for m in cellsMatched]))
								return valueRemoved
		return False

	def nPossibles(self, n):
		"""ref: http://www.simes.clara.co.uk/programs/sudokutechnique9.htm
		reducing candidates rule"""
		for s in self.groups:
			# check that the number of blank cells is greater than n
			blanks = [bc for bc in s.cells if not bc.value]
			l = len(blanks)
			if l <= n: continue
			# get all possible combinations of n unset cells
			for combo in self.permute(0,l,n):
				possibles = sets.Set([])
				# make a list of all the possibles in the cells we're combining
				for i in range(n):
					possibles.union_update(blanks[combo[i]].possibles)
				# remove from the list the possibles values for other cells in the group
				for i in range(l):
					if i in combo: continue
					possibles.difference_update(blanks[i].possibles)
				# if the number of remaining possibles is the same as the number of cells we're combining
				if len(possibles) == n:
					#remove all values but those in possibles from the possibles list of the cells in the combination
					valueRemoved = False
					for cell in [blanks[c] for c in combo]:
						for i in cell.possibles:
							if i not in possibles:
								valueRemoved = True
								cell.possibles.remove(i)
					if valueRemoved:
						self.report.message("the values %s in %s can only be "
								"provided by %s" % (' and '.join([str(j) for j in possibles]), 
								s, ' or '.join([str(blanks[i]) for i in combo])), 
								linenum=True,
								cells=[(blanks[i].row,blanks[i].col) for i in combo]
								) 
						return valueRemoved
		return False

#**************************** class Statistics **************************

class Statistics(object):

	def __init__(self):
		self.ngroups = {}
		self.npossibles = {}
		self.xwings = 0
		self.chains = 0
		self.fragments = 0
		self.impossible = False

	def inc(self, dict, key=None):
		if not key:
			exec("self.%s += 1" % dict)
		else:
			exec("variable = self." + dict)
			if variable.has_key(key): 
				variable[key] += 1
			else: 
				variable[key] = 1

	def classify(self):
		if self.impossible: return "impossible?"
		elif self.xwings or self.chains: return "very hard"
		elif self.ngroups and [l for l in self.ngroups.keys() if l > 2]: return "hard"
		elif self.npossibles and [l for l in self.npossibles.keys() if l > 2]: return "hard"
		elif self.fragments or self.ngroups or self.npossibles: return "medium"
		else: return "easy"

	def __str__(self):
		string = "\nstatistics:"
		string += "\nfragments = %s" % self.fragments
		string += "\nngroups = %s" % self.ngroups
		string += "\nnpossibles = %s" % self.npossibles
		string += "\nxwings = %s" % self.xwings
		string += "\nchains = %s" % self.chains
		string += "\nclassification = %s\n" % self.classify()
		return string


