from wxWin import Table, UndoBuffer
import wx

#****************************** class SudokuTable ******************************

class SudokuTable(Table):

	hilight = wx.Color(0,0,255)
	normal  = wx.Color(0,0,0)
	lolight = wx.Color(255,255,255)

	def __init__(self, parent, size=3, grid=None, cellSize=32):
		self.grid = grid
		Table.__init__(self, parent=parent, size=size, grid=grid, cellSize=cellSize)
		self.readAllValues()
		
		self.redundants = None
		#the set of values to which a cells can be returned by clicking
		self.clickRestore = None
		self.undoBuffer = UndoBuffer(self, self.storeUndo, self.restoreUndo)
		self.clickSetup = False
		self.clearClicked = False
		self.clearRedundant = False
		self.changePossibles = False
		self.selectValue = False
		self.anyValue = False
		self.restoreValue = False
		self.redundantsHighlighted = False
		self.selector = None
		self.clickRow = -1
		self.clickCol = -1
		self.visibleSolution = -1
		self.maxSolutions = 0
		wx.grid.EVT_GRID_CELL_RIGHT_CLICK(self, self.rightClicked)
		wx.grid.EVT_GRID_CELL_LEFT_CLICK(self, self.leftClicked)
		SolutionStep.save = type(grid).listValues
		SolutionStep.restore = type(grid).restoreValues

	def _setGrid(self): pass
	
	def storeUndo(self):
		return (self.grid.listValues(), self.clickRestore)

	def restoreUndo(self, data):
		self.grid.restoreValues(data[0])
		if self.redundantsHighlighted: 
			self.highlightRedundants()
		self.readAllValues()
		self.clickRestore = data[1]
	
	def __getSize(self): return self.grid.size
	size = property(__getSize)

	def open(self, file=None, string=None, xmlFile=None, csvFile=None):
		if string: self.grid.setReader(string=string)
		elif xmlFile: self.grid.setReader(xmlFile=xmlFile)
		elif csvFile: self.grid.setReader(csvFile=csvFile)
		else: self.grid.setReader(file=file)
		if self.grid.size <> self.size:
			self.grid.report.message("sorry only %d by %d grids will fit in this window - (you could start another)" % (self.size*self.size, self.size*self.size))
			return
		self.grid.zeroValues()
		self.grid.read()
		self.SetDefaultCellTextColour(self.normal)
		for i in range(self.size**2):
			for j in range(self.size**2):
				self.SetCellTextColour(i, j, self.normal)
		self.newFile()
		self.readAllValues()
		self.undoBuffer.store()
		self.clickRestore = self.grid.listValues()
	
	def newFile(self):
		"actions to be carried out when a new file is read"
		pass

	def clearAllValues(self):
		"clears all values from the screen and the underlying grid"
		self.grid.zeroValues()
		self.ClearGrid()

	def showThisSolution(self):
		if self.visibleSolution < 0 or \
				self.visibleSolution > len(self.grid.solutions) - 1:
			return
		self.grid.restoreValues(self.grid.solutions[self.visibleSolution])
		self.clickRestore = self.grid.listValues()
		self.SetDefaultCellTextColour(self.normal)
		self.readAllValues()
		self.grid.report.message("showing solution %d of %d%s" % 
			(self.visibleSolution+1, self.grid.nsolutions, 
			self.grid.nsolutions == self.maxSolutions and " (there may be more)" or ""))
		self.undoBuffer.store()

	def __clearCell(self, row, col, redundantsOnly):
		if redundantsOnly and (self.size**2*row + col) not in self.redundants: 
			return
		if self.clickRestore:
			self.SetCellTextColour(row, col, self.lolight)
		else:
			self.SetCellValue(row, col, "")
		self.grid.cell(row, col).value = 0
		if self.redundantsHighlighted: self.highlightRedundants()
		self.Refresh()
		self.undoBuffer.store()

	def readAllValues(self, showingSteps=False):
		"reads values from the underlying grid into the display"
		for i in range(self.size**2):
			for j in range(self.size**2):
				self.SetCellTextColour(i, j, self.normal)
				self.SetCellValue(i,j, str(self.grid.cells[i*self.size**2+j].value or "") )
		self.Refresh()

	def highlightRedundants(self):
		"highlights the cells whose values can be calculated from others"
		self.redundants = self.grid.findRedundants()
		for i in range(self.size**4):
			if i in self.redundants:
				self.SetCellTextColour(i//self.size**2, i%self.size**2, self.hilight)
			elif self.clickRestore and i in self.grid.findZeros():
				self.SetCellTextColour(i//self.size**2, i%self.size**2, self.lolight)
			else:
				self.SetCellTextColour(i//self.size**2, i%self.size**2, self.normal)
		self.redundantsHighlighted = True
		self.Refresh()

	def removeHighlights(self):
		for i in range(self.size**4):
			self.SetDefaultCellTextColour(self.normal)
			if self.clickRestore and i in self.grid.findZeros():
				self.SetCellTextColour(i//self.size**2, i%self.size**2, self.lolight)
			else:
				self.SetCellTextColour(i//self.size**2, i%self.size**2, self.normal)
		self.redundantsHighlighted = False
		self.Refresh()
	
	def leftClicked(self, event):
		(row, col) = (event.GetRow(), event.GetCol())
		if self.clearClicked:
			self.__clearCell(row, col, False)
		elif self.clearRedundant:
			self.__clearCell(row, col, True)
		elif self.changePossibles:
			#self.editPossibles(row, col, event.GetPosition())
			self.editPossibles(row, col)

	def rightClicked(self, event):
		(row, col) = (event.GetRow(), event.GetCol())
		if self.selectValue:
			position = event.GetPosition()
			self.valueSelect(row, col, position)
		elif self.restoreValue:
			self.clickReset(row, col)

	def valueSelect(self, row, col, position):
		"produces a popup menu to allow user to select cell value"
		#save values identifying cell, for use by valueChosen method
		self.clickRow = row
		self.clickCol = col
		if not self.selector: self.selector = wx.Menu()
		self.__clearSelector()
		if self.anyValue:
			choices = range(self.size**2)
		else:
			self.grid.initializeSolveValues()
			choices = self.grid.cells[self.size**2*row+col].possibles
		for value in choices:
			id = self.selector.Append(value, '&'+str(value)).GetId()
			wx.EVT_MENU(self.selector, id, self.valueChosen)
		self.parent.PopupMenu(self.selector,position)

	def __clearSelector(self):
		for i in self.selector.GetMenuItems():
			self.selector.Delete(i.GetId())

	def valueChosen(self, event):
		"sets the value chosen by the user"
		value = event.GetId()
		row, col = self.clickRow, self.clickCol
		self.grid.cells[self.size**2*row+col].value = value
		self.SetCellTextColour(row, col, self.normal)
		self.SetCellValue(row, col, str(value) or "")
		self.undoBuffer.store()

	def clickReset(self, row, col):
		i = self.size**2*row + col
		self.grid.cells[i].value = self.clickRestore[i]
		self.SetCellValue(row, col, 
			self.grid.cells[i].value and str(self.grid.cells[i].value) or "" )
		if self.redundantsHighlighted: self.highlightRedundants()
		self.SetCellTextColour(row, col, self.normal)
		self.Refresh()
		self.undoBuffer.store()

#****************************** class SolutionStep ******************************

class SolutionStep(object):

	# class functions set by the table initializer:
	save = None
	restore = None

	def __init__(self, text, listbox, cells=[]):
		self.text = text
		self.cells = cells
		self.status = self.save(listbox.win.table.grid)
		listbox.list.append(text, clientData=self)

